using System.Diagnostics;
using Jellyfin.Plugin.SmartPlaylist.Lisp;

namespace Jellyfin.Plugin.SmartPlaylist.Lisp.Compiler {
    public class Parser {
        private StringTokenStream _sts;
        public Parser(StringTokenStream tokens) {
            _sts = tokens;
        }
        public Parser(string s) {
            _sts = StringTokenStream.generate(s);
        }

        public Expression parse() {
            Token<string> token = _sts.Get();
            switch (token) {
                case GroupingToken gt:
                    return parse_grouping(gt, gt.closing_value);
                case AtomToken at:
                    return parse_atom(at);
                case SpaceToken sp:
                    return parse();
            }
            return parse();
        }

        Expression parse_string(GroupingToken start, GroupingToken? end) {
            Debug.Assert(end != null);
            Debug.Assert(start.value == end.value);
            Debug.Assert("\"".Contains(start.value));
            string r = "";
            while (_sts.Available() > 0) {
                Token<string> t = _sts.Get();
                if (t.value == end.value) {
                    break;
                }
                r += t.value;
            }
            _sts.Commit();
            return new String(r);
        }

        Expression parse_grouping(GroupingToken start, GroupingToken? end) {
            Debug.Assert(end != null);
            if ("\"".Contains(start.value)) {
                return parse_string(start, end);
            }
            IList<Expression> expressions = new List<Expression>();
            while (_sts.Available() > 0) {
                Token<string> t = _sts.Get();
                if (t.value == end.value) {
                    _sts.Commit();
                    break;
                }
                _sts.Rewind(1);
                expressions.Add(parse());
            }
            return Cons.FromList(expressions);
        }

        Expression parse_atom(AtomToken at) {
            int parsed_value;
            if (int.TryParse(at.value, out parsed_value)) {
                _sts.Commit();
                return new Integer(parsed_value);
            }
            if (at.value.Equals("t")) {
                return Boolean.TRUE;
            }
            if (at.value.Equals("nil")) {
                return Boolean.FALSE;
            }
            _sts.Commit();
            return new Symbol(at.value);
        }
    }
}
